@@ -3,51 +3,60 @@
  *  @ingroup MGparm
  *  @author  Nathan Baker
  *  @brief   Class MGparm methods
- *  @version $Id: mgparm.c,v 1.33 2005/12/07 21:18:37 todd_dolinsky Exp $
+ *  @version $Id$
  *  @attention
  *  @verbatim
  *
  * APBS -- Adaptive Poisson-Boltzmann Solver
  *
- * Nathan A. Baker (baker@biochem.wustl.edu)
- * Dept. of Biochemistry and Molecular Biophysics
- * Center for Computational Biology
- * Washington University in St. Louis
+ *  Nathan A. Baker (nathan.baker@pnnl.gov)
+ *  Pacific Northwest National Laboratory
  *
- * Additional contributing authors listed in the code documentation.
+ *  Additional contributing authors listed in the code documentation.
  *
- * Copyright (c) 2002-2005.  Washington University in St. Louis.
- * All Rights Reserved.
- * Portions Copyright (c) 1999-2002.  The Regents of the University of
- * California.  
- * Portions Copyright (c) 1995.  Michael Holst.
+ * Copyright (c) 2010-2020 Battelle Memorial Institute. Developed at the
+ * Pacific Northwest National Laboratory, operated by Battelle Memorial
+ * Institute, Pacific Northwest Division for the U.S. Department of Energy.
  *
- * This file is part of APBS.
+ * Portions Copyright (c) 2002-2010, Washington University in St. Louis.
+ * Portions Copyright (c) 2002-2020, Nathan A. Baker.
+ * Portions Copyright (c) 1999-2002, The Regents of the University of
+ * California.
+ * Portions Copyright (c) 1995, Michael Holst.
+ * All rights reserved.
  *
- * APBS is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * APBS is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Redistributions of source code must retain the above copyright notice, this
+ * list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU General Public License
- * along with APBS; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ * Redistributions in binary form must reproduce the above copyright notice,
+ * this list of conditions and the following disclaimer in the documentation
+ * and/or other materials provided with the distribution.
+ *
+ * Neither the name of the developer nor the names of its contributors may be
+ * used to endorse or promote products derived from this software without
+ * specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @endverbatim
  */
 
-#include "apbscfg.h"
-#include "apbs/apbs.h"
-#include "apbs/vhal.h"
-#include "apbs/mgparm.h"
-#include "apbs/vstring.h"
+#include "mgparm.h"
 
-VEMBED(rcsid="$Id: mgparm.c,v 1.33 2005/12/07 21:18:37 todd_dolinsky Exp $")
+VEMBED(rcsid="$Id$")
 
 #if !defined(VINLINE_MGPARM)
 
@@ -77,18 +86,6 @@ VPUBLIC double MGparm_getCenterZ(MGparm *thee) {
     VASSERT(thee != VNULL);
     return thee->center[2];
 }
-VPUBLIC double MGparm_getPartOlapCenterShiftX(MGparm *thee) {
-    VASSERT(thee != VNULL);
-    return thee->partOlapCenterShift[0];
-}
-VPUBLIC double MGparm_getPartOlapCenterShiftY(MGparm *thee) {
-    VASSERT(thee != VNULL);
-    return thee->partOlapCenterShift[1];
-}
-VPUBLIC double MGparm_getPartOlapCenterShiftZ(MGparm *thee) {
-    VASSERT(thee != VNULL);
-    return thee->partOlapCenterShift[2];
-}
 VPUBLIC int MGparm_getNx(MGparm *thee) {
     VASSERT(thee != VNULL);
     return thee->dime[0];
@@ -118,18 +115,18 @@ VPUBLIC MGparm* MGparm_ctor(MGparm_CalcType type) {
 
     /* Set up the structure */
     MGparm *thee = VNULL;
-    thee = Vmem_malloc(VNULL, 1, sizeof(MGparm));
+    thee = (MGparm*)Vmem_malloc(VNULL, 1, sizeof(MGparm));
     VASSERT( thee != VNULL);
-    VASSERT( MGparm_ctor2(thee, type) );
+    VASSERT( MGparm_ctor2(thee, type) == VRC_SUCCESS );
 
     return thee;
 }
 
-VPUBLIC int MGparm_ctor2(MGparm *thee, MGparm_CalcType type) {
+VPUBLIC Vrc_Codes MGparm_ctor2(MGparm *thee, MGparm_CalcType type) {
 
     int i;
 
-    if (thee == VNULL) return 0;
+    if (thee == VNULL) return VRC_FAILURE;
 
     for (i=0; i<3; i++) {
         thee->dime[i] = -1;
@@ -146,9 +143,11 @@ VPUBLIC int MGparm_ctor2(MGparm *thee, MGparm_CalcType type) {
     /* *** TYPE 0 PARAMETERS *** */
     thee->nlev = VMGNLEV;
     thee->setnlev = 1;
+    thee->etol = 1.0e-6;
+    thee->setetol = 0;
     thee->setgrid = 0;
     thee->setglen = 0;
-    thee->setgcent = 0;  
+    thee->setgcent = 0;
 
     /* *** TYPE 1 & 2 PARAMETERS *** */
     thee->setcglen = 0;
@@ -162,9 +161,12 @@ VPUBLIC int MGparm_ctor2(MGparm *thee, MGparm_CalcType type) {
     thee->setsize = 0;
     thee->setofrac = 0;
     for (i=0; i<6; i++) thee->partDisjOwnSide[i] = 0;
-	thee->setasync = 0;
+    thee->setasync = 0;
 
-    return 1; 
+    thee->useAqua = 0;
+    thee->setUseAqua = 0;
+
+    return VRC_SUCCESS;
 }
 
 VPUBLIC void MGparm_dtor(MGparm **thee) {
@@ -177,90 +179,95 @@ VPUBLIC void MGparm_dtor(MGparm **thee) {
 
 VPUBLIC void MGparm_dtor2(MGparm *thee) { ; }
 
-VPUBLIC int MGparm_check(MGparm *thee) { 
+VPUBLIC Vrc_Codes MGparm_check(MGparm *thee) {
 
-    int rc, i, tdime[3], ti, tnlev[3], nlev;
+    Vrc_Codes rc;
+    int i, tdime[3], ti, tnlev[3], nlev;
 
-    rc = 1;
+    rc = VRC_SUCCESS;
+
+    Vnm_print(0, "MGparm_check:  checking MGparm object of type %d.\n",
+              thee->type);
 
     /* Check to see if we were even filled... */
     if (!thee->parsed) {
         Vnm_print(2, "MGparm_check:  not filled!\n");
-        return 0;
+        return VRC_FAILURE;
     }
 
     /* Check generic settings */
     if (!thee->setdime) {
         Vnm_print(2, "MGparm_check:  DIME not set!\n");
-        rc = 0;
+        rc = VRC_FAILURE;
     }
     if (!thee->setchgm) {
         Vnm_print(2, "MGparm_check: CHGM not set!\n");
-        return 0;
+        return VRC_FAILURE;
     }
 
 
     /* Check sequential manual & dummy settings */
-    if ((thee->type == MCT_MAN) || (thee->type == MCT_DUM)) {
+    if ((thee->type == MCT_MANUAL) || (thee->type == MCT_DUMMY)) {
         if ((!thee->setgrid) && (!thee->setglen)) {
             Vnm_print(2, "MGparm_check:  Neither GRID nor GLEN set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if ((thee->setgrid) && (thee->setglen)) {
             Vnm_print(2, "MGparm_check:  Both GRID and GLEN set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setgcent) {
             Vnm_print(2, "MGparm_check:  GCENT not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
     }
- 
+
     /* Check sequential and parallel automatic focusing settings */
-    if ((thee->type == MCT_AUT) || (thee->type == MCT_PAR)) {
+    if ((thee->type == MCT_AUTO) || (thee->type == MCT_PARALLEL)) {
         if (!thee->setcglen) {
             Vnm_print(2, "MGparm_check:  CGLEN not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setfglen) {
             Vnm_print(2, "MGparm_check:  FGLEN not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setcgcent) {
             Vnm_print(2, "MGparm_check:  CGCENT not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setfgcent) {
             Vnm_print(2, "MGparm_check:  FGCENT not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
     }
 
     /* Check parallel automatic focusing settings */
-    if (thee->type == MCT_PAR) {
+    if (thee->type == MCT_PARALLEL) {
         if (!thee->setpdime) {
             Vnm_print(2, "MGparm_check:  PDIME not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setrank) {
             Vnm_print(2, "MGparm_check:  PROC_RANK not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setsize) {
             Vnm_print(2, "MGparm_check:  PROC_SIZE not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
         if (!thee->setofrac) {
             Vnm_print(2, "MGparm_check:  OFRAC not set!\n");
-            rc = 0;
+            rc = VRC_FAILURE;
         }
     }
- 
+
     /* Perform a sanity check on nlev and dime, resetting values as necessary */
     if (rc == 1) {
-	/* Calculate the actual number of grid points and nlev to satisfy the
-	 * formula:  n = c * 2^(l+1) + 1, where n is the number of grid points,
-	 * c is an integer, and l is the number of levels */
+    /* Calculate the actual number of grid points and nlev to satisfy the
+     * formula:  n = c * 2^(l+1) + 1, where n is the number of grid points,
+     * c is an integer, and l is the number of levels */
+    if (thee->type != MCT_DUMMY) {
         for (i=0; i<3; i++) {
             /* See if the user picked a reasonable value, if not then fix it */
             ti = thee->dime[i] - 1;
@@ -288,12 +295,22 @@ VPUBLIC int MGparm_check(MGparm *thee) {
                     if (ti < 1) ti = 1;
                     tdime[i] = ti*(int)(VPOW(2.,(VMGNLEV+1))) + 1;
                     tnlev[i] = 4;
-                    Vnm_print(2, "NOsh:  Reset dime[%d] to %d and \
-(nlev = %d).\n", i, tdime[i], VMGNLEV);
+                    Vnm_print(2, "NOsh:  Reset dime[%d] to %d and (nlev = %d).\n", i, tdime[i], VMGNLEV);
                 }
             }
         }
-	/* The actual number of levels we'll be using is the smallest number of
+    } else { /* We are a dummy calculation, but we still need positive numbers of points */
+        for (i=0; i<3; i++) {
+            tnlev[i] = thee->nlev;
+            tdime[i] = thee->dime[i];
+                if (thee->dime[i] <= 0) {
+                    Vnm_print(2, "NOsh:  Resetting dime[%d] from %d to 3.\n", i, thee->dime[i]);
+                    thee->dime[i] = 3;
+            }
+        }
+    }
+
+    /* The actual number of levels we'll be using is the smallest number of
          * possible levels in any dimensions */
         nlev = VMIN2(tnlev[0], tnlev[1]);
         nlev = VMIN2(nlev, tnlev[2]);
@@ -301,9 +318,20 @@ VPUBLIC int MGparm_check(MGparm *thee) {
         Vnm_print(0, "NOsh:  nlev = %d, dime = (%d, %d, %d)\n", nlev, tdime[0],
           tdime[1], tdime[2]);
         thee->nlev = nlev;
+        if (thee->nlev <= 0) {
+            Vnm_print(2, "MGparm_check:  illegal nlev (%d); check your grid dimensions!\n", thee->nlev);
+            rc = VRC_FAILURE;
+        }
+        if (thee->nlev < 2) {
+            Vnm_print(2, "MGparm_check:  you're using a very small nlev (%d) and therefore\n", thee->nlev);
+            Vnm_print(2, "MGparm_check:  will not get the optimal performance of the multigrid\n");
+            Vnm_print(2, "MGparm_check:  algorithm.  Please check your grid dimensions.\n");
+        }
         for (i=0; i<3; i++) thee->dime[i] = tdime[i];
     }
 
+    if (!thee->setUseAqua) thee->useAqua = 0;
+
     return rc;
 }
 
@@ -314,7 +342,7 @@ VPUBLIC void MGparm_copy(MGparm *thee, MGparm *parm) {
     VASSERT(thee != VNULL);
     VASSERT(parm != VNULL);
 
-    
+
     thee->type = parm->type;
     thee->parsed = parm->parsed;
 
@@ -323,10 +351,13 @@ VPUBLIC void MGparm_copy(MGparm *thee, MGparm *parm) {
     thee->setdime = parm->setdime;
     thee->chgm = parm->chgm;
     thee->setchgm = parm->setchgm;
+    thee->chgs = parm->chgs;
 
     /* *** TYPE 0 PARMS *** */
     thee->nlev = parm->nlev;
     thee->setnlev = parm->setnlev;
+    thee->etol = parm->etol;
+    thee->setetol = parm->setetol;
     for (i=0; i<3; i++) thee->grid[i] = parm->grid[i];
     thee->setgrid = parm->setgrid;
     for (i=0; i<3; i++) thee->glen[i] = parm->glen[i];
@@ -351,16 +382,12 @@ VPUBLIC void MGparm_copy(MGparm *thee, MGparm *parm) {
     thee->fcentmol = parm->fcentmol;
 
     /* *** TYPE 2 PARMS *** */
-    for (i=0; i<3; i++) 
-      thee->partDisjCenterShift[i] = parm->partDisjCenterShift[i];
-    for (i=0; i<3; i++) 
+    for (i=0; i<3; i++)
+      thee->partDisjCenter[i] = parm->partDisjCenter[i];
+    for (i=0; i<3; i++)
       thee->partDisjLength[i] = parm->partDisjLength[i];
-    for (i=0; i<6; i++) 
+    for (i=0; i<6; i++)
       thee->partDisjOwnSide[i] = parm->partDisjOwnSide[i];
-    for (i=0; i<3; i++) 
-      thee->partOlapCenterShift[i] = parm->partOlapCenterShift[i];
-    for (i=0; i<3; i++) 
-      thee->partOlapLength[i] = parm->partOlapLength[i];
     for (i=0; i<3; i++) thee->pdime[i] = parm->pdime[i];
     thee->setpdime = parm->setpdime;
     thee->proc_rank = parm->proc_rank;
@@ -369,12 +396,20 @@ VPUBLIC void MGparm_copy(MGparm *thee, MGparm *parm) {
     thee->setsize = parm->setsize;
     thee->ofrac = parm->ofrac;
     thee->setofrac = parm->setofrac;
-	thee->setasync = parm->setasync;
-	thee->async = parm->async;
+    thee->setasync = parm->setasync;
+    thee->async = parm->async;
 
+    thee->nonlintype = parm->nonlintype;
+    thee->setnonlintype = parm->setnonlintype;
+
+    thee->method = parm->method;
+    thee->method = parm->method;
+
+    thee->useAqua = parm->useAqua;
+    thee->setUseAqua = parm->setUseAqua;
 }
 
-VPRIVATE int MGparm_parseDIME(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseDIME(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     int ti;
@@ -383,35 +418,35 @@ VPRIVATE int MGparm_parseDIME(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%d", &ti) == 0){
         Vnm_print(2, "parseMG:  Read non-integer (%s) while parsing DIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->dime[0] = ti;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing DIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->dime[1] = ti;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing DIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->dime[2] = ti;
     thee->setdime = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseCHGM(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseCHGM(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
-    int ti;
+    Vchrg_Meth ti;
 
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
-    if (sscanf(tok, "%d", &ti) == 1) {
+    if (sscanf(tok, "%d", (int*)(&ti)) == 1) {
         thee->chgm = ti;
         thee->setchgm = 1;
         Vnm_print(2, "NOsh:  Warning -- parsed deprecated statment \"chgm %d\".\n", ti);
@@ -423,33 +458,40 @@ VPRIVATE int MGparm_parseCHGM(MGparm *thee, Vio *sock) {
             case VCM_BSPL2:
                 Vnm_print(2, "spl2");
                 break;
+            case VCM_BSPL4:
+                Vnm_print(2, "spl4");
+                break;
             default:
                 Vnm_print(2, "UNKNOWN");
                 break;
         }
         Vnm_print(2, "\" instead!\n");
-        return 1;
+        return VRC_SUCCESS;
     } else if (Vstring_strcasecmp(tok, "spl0") == 0) {
         thee->chgm = VCM_TRIL;
         thee->setchgm = 1;
-        return 1;
+        return VRC_SUCCESS;
     } else if (Vstring_strcasecmp(tok, "spl2") == 0) {
         thee->chgm = VCM_BSPL2;
         thee->setchgm = 1;
-        return 1;
+        return VRC_SUCCESS;
+    } else if (Vstring_strcasecmp(tok, "spl4") == 0) {
+        thee->chgm = VCM_BSPL4;
+        thee->setchgm = 1;
+        return VRC_SUCCESS;
     } else {
         Vnm_print(2, "NOsh:  Unrecognized parameter (%s) when parsing \
 chgm!\n", tok);
-        return -1;
+        return VRC_WARNING;
     }
-    return -1;
+    return VRC_WARNING;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseNLEV(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseNLEV(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     int ti;
@@ -458,17 +500,40 @@ VPRIVATE int MGparm_parseNLEV(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing NLEV \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->nlev = ti;
     thee->setnlev = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseGRID(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseETOL(MGparm *thee, Vio *sock) {
+
+    char tok[VMAX_BUFSIZE];
+    double tf;
+
+    VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
+    if (sscanf(tok, "%lf", &tf) == 0) {
+        Vnm_print(2, "NOsh:  Read non-float (%s) while parsing etol \
+keyword!\n", tok);
+        return VRC_WARNING;
+    } else if (tf <= 0.0) {
+        Vnm_print(2, "parseMG:  etol must be greater than 0!\n");
+        return VRC_WARNING;
+    } else thee->etol = tf;
+    thee->setetol = 1;
+    return VRC_SUCCESS;
+
+    VERROR1:
+        Vnm_print(2, "parseMG:  ran out of tokens!\n");
+        return VRC_WARNING;
+}
+
+
+VPRIVATE Vrc_Codes MGparm_parseGRID(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -477,29 +542,29 @@ VPRIVATE int MGparm_parseGRID(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GRID \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->grid[0] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GRID \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->grid[1] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GRID \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->grid[2] = tf;
     thee->setgrid = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseGLEN(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseGLEN(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -508,29 +573,44 @@ VPRIVATE int MGparm_parseGLEN(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->glen[0] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->glen[1] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing GLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->glen[2] = tf;
     thee->setglen = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseGCENT(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseGAMMA(MGparm *thee, Vio *sock) {
+
+    char tok[VMAX_BUFSIZE];
+
+    VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
+    Vnm_print(2, "parseMG:  GAMMA keyword deprecated!\n");
+    Vnm_print(2, "parseMG:  If you are using PyMOL or VMD and still seeing this message,\n");
+    Vnm_print(2, "parseMG:  please contact the developers of those programs regarding this message.\n");
+    return VRC_SUCCESS;
+
+VERROR1:
+    Vnm_print(2, "parseMG:  ran out of tokens!\n");
+    return VRC_WARNING;
+}
+
+VPRIVATE Vrc_Codes MGparm_parseGCENT(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -545,15 +625,17 @@ VPRIVATE int MGparm_parseGCENT(MGparm *thee, Vio *sock) {
             if (sscanf(tok, "%d", &ti) == 0) {
                 Vnm_print(2, "NOsh:  Read non-int (%s) while parsing \
 GCENT MOL keyword!\n", tok);
-                return -1;
+                return VRC_WARNING;
             } else {
-                thee->cmeth = MCM_MOL;
-                thee->centmol = ti;
+                thee->cmeth = MCM_MOLECULE;
+                /* Subtract 1 here to convert user numbering (1, 2, 3, ...) into
+                array index */
+                thee->centmol = ti - 1;
             }
         } else {
             Vnm_print(2, "NOsh:  Unexpected keyword (%s) while parsing \
 GCENT!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
     } else {
         thee->center[0] = tf;
@@ -561,26 +643,26 @@ GCENT!\n", tok);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 GCENT keyword!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
         thee->center[1] = tf;
         VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 GCENT keyword!\n", tok);
-            return -1;
-        } 
+            return VRC_WARNING;
+        }
         thee->center[2] = tf;
     }
     thee->setgcent = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseCGLEN(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseCGLEN(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -589,29 +671,29 @@ VPRIVATE int MGparm_parseCGLEN(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing CGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->cglen[0] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing CGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->cglen[1] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing CGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->cglen[2] = tf;
     thee->setcglen = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseFGLEN(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseFGLEN(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -620,29 +702,29 @@ VPRIVATE int MGparm_parseFGLEN(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing FGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->fglen[0] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing FGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->fglen[1] = tf;
     VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-float (%s) while parsing FGLEN \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else thee->fglen[2] = tf;
     thee->setfglen = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseCGCENT(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseCGCENT(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -657,15 +739,17 @@ VPRIVATE int MGparm_parseCGCENT(MGparm *thee, Vio *sock) {
             if (sscanf(tok, "%d", &ti) == 0) {
                 Vnm_print(2, "NOsh:  Read non-int (%s) while parsing \
 CGCENT MOL keyword!\n", tok);
-                return -1;
+                return VRC_WARNING;
             } else {
-                thee->ccmeth = MCM_MOL;
-                thee->ccentmol = ti;
+                thee->ccmeth = MCM_MOLECULE;
+                /* Subtract 1 here to convert user numbering (1, 2, 3, ...) into
+                array index */
+                thee->ccentmol = ti - 1;
             }
         } else {
             Vnm_print(2, "NOsh:  Unexpected keyword (%s) while parsing \
 CGCENT!\n", tok);
-            return -1;
+            return VRC_WARNING;
             }
     } else {
         thee->ccenter[0] = tf;
@@ -673,26 +757,26 @@ CGCENT!\n", tok);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 CGCENT keyword!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
         thee->ccenter[1] = tf;
         VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 CGCENT keyword!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
         thee->ccenter[2] = tf;
     }
     thee->setcgcent = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseFGCENT(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseFGCENT(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -707,15 +791,17 @@ VPRIVATE int MGparm_parseFGCENT(MGparm *thee, Vio *sock) {
             if (sscanf(tok, "%d", &ti) == 0) {
                  Vnm_print(2, "NOsh:  Read non-int (%s) while parsing \
 FGCENT MOL keyword!\n", tok);
-                 return -1;
+                 return VRC_WARNING;
             } else {
-                thee->fcmeth = MCM_MOL;
-                thee->fcentmol = ti;
+                thee->fcmeth = MCM_MOLECULE;
+                /* Subtract 1 here to convert user numbering (1, 2, 3, ...) into
+                array index */
+                thee->fcentmol = ti - 1;
             }
         } else {
             Vnm_print(2, "NOsh:  Unexpected keyword (%s) while parsing \
 FGCENT!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
     } else {
         thee->fcenter[0] = tf;
@@ -723,26 +809,26 @@ FGCENT!\n", tok);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 FGCENT keyword!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
         thee->fcenter[1] = tf;
         VJMPERR1(Vio_scanf(sock, "%s", tok) == 1);
         if (sscanf(tok, "%lf", &tf) == 0) {
             Vnm_print(2, "NOsh:  Read non-float (%s) while parsing \
 FGCENT keyword!\n", tok);
-            return -1;
+            return VRC_WARNING;
         }
         thee->fcenter[2] = tf;
     }
     thee->setfgcent = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parsePDIME(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parsePDIME(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     int ti;
@@ -752,7 +838,7 @@ VPRIVATE int MGparm_parsePDIME(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing PDIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else {
         thee->pdime[0] = ti;
     }
@@ -760,7 +846,7 @@ keyword!\n", tok);
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing PDIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else {
         thee->pdime[1] = ti;
     }
@@ -768,19 +854,19 @@ keyword!\n", tok);
     if (sscanf(tok, "%d", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing PDIME \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     } else {
         thee->pdime[2] = ti;
     }
     thee->setpdime = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseOFRAC(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseOFRAC(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     double tf;
@@ -789,18 +875,18 @@ VPRIVATE int MGparm_parseOFRAC(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%lf", &tf) == 0) {
         Vnm_print(2, "NOsh:  Read non-int (%s) while parsing OFRAC \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     }
     thee->ofrac = tf;
     thee->setofrac = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPRIVATE int MGparm_parseASYNC(MGparm *thee, Vio *sock) {
+VPRIVATE Vrc_Codes MGparm_parseASYNC(MGparm *thee, Vio *sock) {
 
     char tok[VMAX_BUFSIZE];
     int ti;
@@ -809,35 +895,48 @@ VPRIVATE int MGparm_parseASYNC(MGparm *thee, Vio *sock) {
     if (sscanf(tok, "%i", &ti) == 0) {
         Vnm_print(2, "NOsh:  Read non-integer (%s) while parsing ASYNC \
 keyword!\n", tok);
-        return -1;
+        return VRC_WARNING;
     }
     thee->async = ti;
     thee->setasync = 1;
-    return 1;
+    return VRC_SUCCESS;
 
     VERROR1:
         Vnm_print(2, "parseMG:  ran out of tokens!\n");
-        return -1;
+        return VRC_WARNING;
 }
 
-VPUBLIC int MGparm_parseToken(MGparm *thee, char tok[VMAX_BUFSIZE], 
+VPRIVATE Vrc_Codes MGparm_parseUSEAQUA(MGparm *thee, Vio *sock) {
+    Vnm_print(0, "NOsh: parsed useaqua\n");
+    thee->useAqua = 1;
+    thee->setUseAqua = 1;
+    return VRC_SUCCESS;
+}
+
+VPUBLIC Vrc_Codes MGparm_parseToken(MGparm *thee, char tok[VMAX_BUFSIZE],
   Vio *sock) {
 
     if (thee == VNULL) {
-        Vnm_print(2, "parseMG:  got NULL thee!\n"); 
-        return -1;
+        Vnm_print(2, "parseMG:  got NULL thee!\n");
+        return VRC_WARNING;
     }
     if (sock == VNULL) {
-        Vnm_print(2, "parseMG:  got NULL socket!\n"); 
-        return -1;
+        Vnm_print(2, "parseMG:  got NULL socket!\n");
+        return VRC_WARNING;
     }
 
+    Vnm_print(0, "MGparm_parseToken:  trying %s...\n", tok);
+
+
     if (Vstring_strcasecmp(tok, "dime") == 0) {
         return MGparm_parseDIME(thee, sock);
     } else if (Vstring_strcasecmp(tok, "chgm") == 0) {
         return MGparm_parseCHGM(thee, sock);
     } else if (Vstring_strcasecmp(tok, "nlev") == 0) {
+        Vnm_print(2, "Warning: The 'nlev' keyword is now deprecated!\n");
         return MGparm_parseNLEV(thee, sock);
+    } else if (Vstring_strcasecmp(tok, "etol") == 0) {
+        return MGparm_parseETOL(thee, sock);
     } else if (Vstring_strcasecmp(tok, "grid") == 0) {
         return MGparm_parseGRID(thee, sock);
     } else if (Vstring_strcasecmp(tok, "glen") == 0) {
@@ -858,11 +957,15 @@ VPUBLIC int MGparm_parseToken(MGparm *thee, char tok[VMAX_BUFSIZE],
         return MGparm_parseOFRAC(thee, sock);
     } else if (Vstring_strcasecmp(tok, "async") == 0) {
         return MGparm_parseASYNC(thee, sock);
+    } else if (Vstring_strcasecmp(tok, "gamma") == 0) {
+        return MGparm_parseGAMMA(thee, sock);
+    } else if (Vstring_strcasecmp(tok, "useaqua") == 0) {
+        return MGparm_parseUSEAQUA(thee, sock);
     } else {
         Vnm_print(2, "parseMG:  Unrecognized keyword (%s)!\n", tok);
-        return -1;
+        return VRC_WARNING;
     }
-   
-    return 0;
+
+    return VRC_FAILURE;
 
 }
