@@ -2,50 +2,18 @@
  *  @file    analysis.c
  *  @author  Nathan Baker
  *  @brief   Program that computes various analyses for a scalar field
- *  @version $Id: analysis.c,v 1.3 2005/12/07 21:18:41 todd_dolinsky Exp $
- *
- *  @attention
- *  @verbatim
- *
- * APBS -- Adaptive Poisson-Boltzmann Solver
- *
- * Nathan A. Baker (baker@biochem.wustl.edu)
- * Dept. of Biochemistry and Molecular Biophysics
- * Center for Computational Biology
- * Washington University in St. Louis
- *
- * Additional contributing authors listed in the code documentation.
- *
- * Copyright (c) 2002-2005.  Washington University in St. Louis.
- * All Rights Reserved.
- * Portions Copyright (c) 1999-2002.  The Regents of the University of
- * California.  
- * Portions Copyright (c) 1995.  Michael Holst.
- *
- * This file is part of APBS.
- *
- * APBS is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * APBS is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with APBS; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
- *
- * @endverbatim
+ *  @version $Id$
+ */
+
+/*
+ *  Last update: 08/29/2016 by Leighton Wilson
+ *  Description: Added ability to read in binary DX files as input
  */
 
 
-#include "apbscfg.h"
-#include "apbs/apbs.h"  
+#include "apbs.h"
 
-VEMBED(rcsid="$Id: analysis.c,v 1.3 2005/12/07 21:18:41 todd_dolinsky Exp $")
+VEMBED(rcsid="$Id$")
 
 /**
  * @brief  Prints usage information and exits
@@ -57,7 +25,8 @@ A program to calculate various metrics for a scalar field\n\
   Usage:  analysis <req args> [opts]\n\
 where <req args> are the required arguments:\n\
   --format=<format>  The input file format.  Acceptable values include\n\
-       dx  OpenDX format\n\
+       dx: standard OpenDX format\n\
+       dxbin: binary OpenDX format\n\
   --scalar=<path>  The path to the scalar data file\n\
 and where [opts] are the options:\n\
   --help  Print this message\n\
@@ -74,12 +43,12 @@ and where [opts] are the options:\n\
 /**
  * @brief  Read a grid
  * @param  grid  Pointer to Vgrid file to be read
- * @param  path  Path to read from 
+ * @param  path  Path to read from
  * @param  format  Format to read
  * @return 1 if successful, 0 otherwise */
 int readGrid(Vgrid **grid, char *path, Vdata_Format format) {
 
-    *grid = Vgrid_ctor(0, 0, 0, 
+    *grid = Vgrid_ctor(0, 0, 0,
             0.0, 0.0, 0.0,
             0.0, 0.0, 0.0,
             VNULL);
@@ -88,6 +57,9 @@ int readGrid(Vgrid **grid, char *path, Vdata_Format format) {
         case VDF_DX:
             return Vgrid_readDX(*grid, "FILE", "ASC", VNULL, path);
             break;
+        case VDF_DXBIN:
+            return Vgrid_readDXBIN(*grid, "FILE", "ASC", VNULL, path);
+            break;
         case VDF_UHBD:
             Vnm_print(2, "Sorry, UHBD input not supported yet!\n");
             return 0;
@@ -123,7 +95,7 @@ int main(int argc, char **argv) {
     Vdata_Format format;
     int gotFormat = 0;
     char *tstr, *targ;
- 
+
     /* *************** CHECK INVOCATION ******************* */
     Vio_start();
     /* Parse args */
@@ -136,6 +108,9 @@ int main(int argc, char **argv) {
             if (strcmp(tstr, "dx") == 0) {
                 format = VDF_DX;
                 gotFormat = 1;
+            } else if (strcmp(tstr, "dxbin") == 0) {
+                format = VDF_DXBIN;
+                gotFormat = 1;
             } else {
                 Vnm_print(2, "Error!  Unknown format (%s)!\n", tstr);
                 usage(2);
@@ -166,7 +141,10 @@ int main(int argc, char **argv) {
     } else {
         switch (format) {
             case VDF_DX:
-                Vnm_print(1, "format:  OpenDX\n");
+                Vnm_print(1, "format:  standard OpenDX\n");
+                break;
+            case VDF_DXBIN:
+                Vnm_print(1, "format:  binary OpenDX\n");
                 break;
             case VDF_UHBD:
                 Vnm_print(1, "format:  UHBD\n");
@@ -195,7 +173,7 @@ int main(int argc, char **argv) {
         Vnm_print(2, "Error reading scalar data set!\n");
         return 2;
     }
-    Vnm_print(1, "Read %d x %d x %d grid.\n", 
+    Vnm_print(1, "Read %d x %d x %d grid.\n",
             scalar->nx, scalar->ny, scalar->nz);
 
     /* Read mask */
@@ -205,16 +183,16 @@ int main(int argc, char **argv) {
             Vnm_print(2, "Error reading mask data set!\n");
             return 2;
         }
-        Vnm_print(1, "Read %d x %d x %d grid.\n", 
+        Vnm_print(1, "Read %d x %d x %d grid.\n",
                 mask->nx, mask->ny, mask->nz);
     }
 
     /* Calculate relative L2 norm of difference */
     Vnm_print(1, "Calculating metrics...\n");
-    nx = scalar->nx; ny = scalar->ny; nz = scalar->nz; 
-    hx = scalar->hx; hy = scalar->hy; hzed = scalar->hzed; 
+    nx = scalar->nx; ny = scalar->ny; nz = scalar->nz;
+    hx = scalar->hx; hy = scalar->hy; hzed = scalar->hzed;
     dvol = (hx*hy*hzed);
-    xmin = scalar->xmin; ymin = scalar->ymin; zmin = scalar->zmin; 
+    xmin = scalar->xmin; ymin = scalar->ymin; zmin = scalar->zmin;
     norm_L1 = 0; norm_L2 = 0; snorm_H1 = 0; norm_H1 = 0;
     haveMaxS = 0; haveMinS = 0; haveMaxG2 = 0; haveMinG2 = 0;
     svol = 0; gvol = 0;
@@ -233,7 +211,7 @@ int main(int argc, char **argv) {
                     gval2 = VSQR(gval[0]) + VSQR(gval[1]) + VSQR(gval[2]);
                 } else gval2 = 0.0;
                 if (gotMask) onGridS = Vgrid_value(mask, pt, &mval);
-                else mval = 1.0; 
+                else mval = 1.0;
 
                 /* Max/min */
                 if (mval > 0) {
@@ -293,35 +271,35 @@ int main(int argc, char **argv) {
     snorm_H1 = VSQRT(snorm_H1*dvol);
     norm_H1 = VSQRT(VSQR(snorm_H1)+VSQR(norm_L2));
 
-    Vnm_print(1, "Volume used to calculate L2 and L1 norms = %1.12E\n", 
+    Vnm_print(1, "Volume used to calculate L2 and L1 norms = %1.12E\n",
             svol);
-    Vnm_print(1, "Volume used to calculate H1 norms        = %1.12E\n", 
+    Vnm_print(1, "Volume used to calculate H1 norms        = %1.12E\n",
             gvol);
-    Vnm_print(1, "Max scalar value                         = %1.12E\n", 
+    Vnm_print(1, "Max scalar value                         = %1.12E\n",
             maxS);
-    Vnm_print(1, "Max scalar value location                = (%4.3f, %4.3f, %4.3f)\n", 
+    Vnm_print(1, "Max scalar value location                = (%4.3f, %4.3f, %4.3f)\n",
             maxSpt[0], maxSpt[1], maxSpt[2]);
-    Vnm_print(1, "Min scalar value                         = %1.12E\n", 
+    Vnm_print(1, "Min scalar value                         = %1.12E\n",
             minS);
-    Vnm_print(1, "Min scalar value location                = (%4.3f, %4.3f, %4.3f)\n", 
+    Vnm_print(1, "Min scalar value location                = (%4.3f, %4.3f, %4.3f)\n",
             minSpt[0], minSpt[1], minSpt[2]);
-    Vnm_print(1, "Max gradient-squared value               = %1.12E\n", 
+    Vnm_print(1, "Max gradient-squared value               = %1.12E\n",
             maxG2);
-    Vnm_print(1, "Max gradient-squared value location      = (%4.3f, %4.3f, %4.3f)\n", 
+    Vnm_print(1, "Max gradient-squared value location      = (%4.3f, %4.3f, %4.3f)\n",
             maxG2pt[0], maxG2pt[1], maxG2pt[2]);
-    Vnm_print(1, "Min gradient-squared value               = %1.12E\n", 
+    Vnm_print(1, "Min gradient-squared value               = %1.12E\n",
             minG2);
-    Vnm_print(1, "Min gradient-squared value location      = (%4.3f, %4.3f, %4.3f)\n", 
+    Vnm_print(1, "Min gradient-squared value location      = (%4.3f, %4.3f, %4.3f)\n",
             minG2pt[0], minG2pt[1], minG2pt[2]);
-    Vnm_print(1, "L2 norm                                  = %1.12E\n", 
+    Vnm_print(1, "L2 norm                                  = %1.12E\n",
             norm_L2);
-    Vnm_print(1, "L1 norm                                  = %1.12E\n", 
+    Vnm_print(1, "L1 norm                                  = %1.12E\n",
             norm_L1);
-    Vnm_print(1, "Linf norm                                = %1.12E\n", 
+    Vnm_print(1, "Linf norm                                = %1.12E\n",
             norm_Linf);
-    Vnm_print(1, "H1 semi-norm                             = %1.12E\n", 
+    Vnm_print(1, "H1 semi-norm                             = %1.12E\n",
             snorm_H1);
-    Vnm_print(1, "H1 norm                                  = %1.12E\n", 
+    Vnm_print(1, "H1 norm                                  = %1.12E\n",
             norm_H1);
 
     return 0;
