@@ -2,47 +2,15 @@
  *  @file    smooth.c
  *  @author  Nathan Baker
  *  @brief   Convolve grid data with various filters
- *  @version $Id: smooth.c,v 1.7 2005/12/07 21:18:41 todd_dolinsky Exp $
- *
- *  @attention
- *  @verbatim
- *
- * APBS -- Adaptive Poisson-Boltzmann Solver
- *
- * Nathan A. Baker (baker@biochem.wustl.edu)
- * Dept. of Biochemistry and Molecular Biophysics
- * Center for Computational Biology
- * Washington University in St. Louis
- *
- * Additional contributing authors listed in the code documentation.
- *
- * Copyright (c) 2002-2005.  Washington University in St. Louis.
- * All Rights Reserved.
- * Portions Copyright (c) 1999-2002.  The Regents of the University of
- * California.  
- * Portions Copyright (c) 1995.  Michael Holst.
- *
- * This file is part of APBS.
- *
- * APBS is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * APBS is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with APBS; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
- *
- * @endverbatim
+ *  @version $Id$
  */
 
-#include "apbscfg.h"
-#include "apbs/apbs.h"  
+/*                                                                                           
+ *  Last update: 08/29/2016 by Leighton Wilson                                               
+ *  Description: Added ability to read in and output binary DX files                           
+ */ 
+
+#include "apbs.h"
 
 #define IJK(i,j,k)  (((k)*(nx)*(ny))+((j)*(nx))+(i))
 #define ERRRC 2
@@ -52,7 +20,7 @@ typedef enum Smooth_Filter {
 } Smooth_Filter;
 
 
-VEMBED(rcsid="$Id: smooth.c,v 1.7 2005/12/07 21:18:41 todd_dolinsky Exp $")
+VEMBED(rcsid="$Id$")
 
 int gaussian(Vgrid *grid, double stddev, double bandwidth);
 
@@ -66,7 +34,9 @@ int usage(int rc) {
     where <args> is a list of the following arguments:\n\
       REQUIRED GENERAL ARGUMENTS:\n\
       --format=<format>  where <format> specifies the data format and is one\n\
-                          of the following: dx (OpenDX)\n\
+                          of the following:\n\
+                                dx (standard OpenDX)\n\
+                                dxbin (binary OpenDX)\n\
       --input=<file>     where <file> is the input mesh data in the\n\
                          specified format\n\
       --output=<file>    where <file> is the output mesh data in the\n\
@@ -83,7 +53,7 @@ int usage(int rc) {
     Vnm_print(2, usage);
 
     exit(rc);
- 
+
     return 0;
 }
 
@@ -112,7 +82,10 @@ int main(int argc, char **argv) {
     for (i=1; i<argc; i++) {
         Vnm_print(1, "Parsing: %s...\n", argv[i]);
         if (strstr(argv[i], "--format") != NULL) {
-            if (strstr(argv[i], "dx") != NULL) {
+            if (strstr(argv[i], "dxbin") != NULL) {
+                gotFormat = 1;
+                format = VDF_DXBIN;
+            } else if (strstr(argv[i], "dx") != NULL) {
                 gotFormat = 1;
                 format = VDF_DX;
             } else {
@@ -146,7 +119,7 @@ int main(int argc, char **argv) {
                 usage(2);
             }
         } else if (strstr(argv[i], "--bandwidth") != NULL) {
-            if (sscanf(argv[i], "--bandwidth=%lf", &bandwidth) == 1) 
+            if (sscanf(argv[i], "--bandwidth=%lf", &bandwidth) == 1)
                 gotBandwidth = 1;
             else {
                 Vnm_print(2, "Error:  %s\n", argv[i]);
@@ -160,7 +133,7 @@ int main(int argc, char **argv) {
     if (!gotFormat) {
         Vnm_print(2, "Error:  --format not specified!\n");
         usage(2);
-    } 
+    }
     if (!gotInPath) {
         Vnm_print(2, "Error:  --input not specified!\n");
         usage(2);
@@ -189,7 +162,13 @@ int main(int argc, char **argv) {
     grid = Vgrid_ctor(0, 0, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, VNULL);
     if (format == VDF_DX) {
         if (!Vgrid_readDX(grid, "FILE", "ASC", VNULL, inPath)) {
-            Vnm_print(2, "main:  Problem reading OpenDX-format grid from %s\n",
+            Vnm_print(2, "main:  Problem reading standard OpenDX-format grid from %s\n",
+              inPath);
+            return ERRRC;
+        }
+    } else if (format == VDF_DXBIN) {
+        if (!Vgrid_readDXBIN(grid, "FILE", "ASC", VNULL, inPath)) {
+            Vnm_print(2, "main:  Problem reading binary OpenDX-format grid from %s\n",
               inPath);
             return ERRRC;
         }
@@ -208,9 +187,12 @@ int main(int argc, char **argv) {
 
     /* *************** READ DATA ******************* */
     Vnm_print(1, "main:  Writing data to %s...\n", outPath);
-    if (format == VDF_DX) 
+    if (format == VDF_DX)
       Vgrid_writeDX(grid, "FILE", "ASC", VNULL, outPath, "Smoothed data",
         VNULL);
+    else if (format == VDF_DXBIN)
+      Vgrid_writeDXBIN(grid, "FILE", "ASC", VNULL, outPath, "Smoothed data",
+        VNULL);
 
     return 0;
 }
@@ -236,9 +218,13 @@ int gaussian(Vgrid *grid, double stddev, double bandwidth) {
     iband = (int)(stddev*bandwidth/hx);
     jband = (int)(stddev*bandwidth/hy);
     kband = (int)(stddev*bandwidth/hzed);
+    /* Special handling for iband, jband and kband, they are non-zero positive integers */
+    if (iband == 0) iband = 1;
+    if (jband == 0) jband = 1;
+    if (kband == 0) kband = 1;
     Vnm_print(1, "Bandwidth converted to %d x %d x %d grid units.\n",
       iband, jband, kband);
-    Vnm_print(1, "This means any non-zero data within (%g, %g, %g) of the\n", 
+    Vnm_print(1, "This means any non-zero data within (%g, %g, %g) of the\n",
       (iband+1)*hx, (jband+1)*hy, (kband+1)*hzed);
     Vnm_print(1, "domain boundary will be convolved differently.\n");
 
@@ -251,6 +237,10 @@ int gaussian(Vgrid *grid, double stddev, double bandwidth) {
     oldData = grid->data;
     newData = Vmem_malloc(VNULL, (nx*ny*nz), sizeof(double));
 
+    /* Copy over old data.  All but the boundary values will be replaced in the next step so this is more copying than is strictly
+       necessary... */
+    for (i=0; i<(nx*ny*nz); i++) newData[i] = oldData[i];
+
     /* Apply filter */
     for (k=1; k<(nz-1); k++) {
         kkmin = VMAX2(1, (k - kband));
